@@ -1,6 +1,9 @@
 import json
 import tempfile
 import os
+import time
+import boto.ec2
+import boto.ec2.cloudwatch
 
 import yaml
 import paramiko
@@ -40,6 +43,16 @@ puts nodeinfo.to_json""" % (env, tier)
         output = Fork.check_output(["knife", "role", "list"], stderr=None)
         return output.split("\n")
 
+    def envs(__self__):
+        output = Fork.check_output(["knife", "environment", "list", "-f", "json"], stderr=None)
+        info = json.loads(output)
+        return info
+
+    def nodes(self, environment):
+        output = Fork.check_output(["knife", "node", "list", "-E", environment, "-f", "json"], stderr=None)
+        info = json.loads(output)
+        return info
+
     def describe(self, environment):
         output = Fork.check_output(["knife", "environment", "show", environment, "-f", "json"], stderr=None)
         info = json.loads(output)
@@ -84,3 +97,34 @@ puts nodeinfo.to_json""" % (env, tier)
         os.unlink(temp)
         return stdout.read()
 
+
+    def reap_instances(self):
+        envs = self.envs()
+        for env_name in envs:
+            if (env_name == "development" or env_name == "_default"):
+                continue
+            print "Checking reap instances for env: " + str(env_name)
+            creds = self.get_credentials(env_name)
+            conn = boto.ec2.connect_to_region("us-east-1",
+                                              aws_access_key_id=creds["access"],
+                                              aws_secret_access_key=creds["secret"])
+
+            cw_obj = boto.ec2.cloudwatch.connect_to_region('us-east-1')
+            nodes = self.nodes(env_name)
+            for node in nodes:
+                i = conn.get_only_instances(node)
+                if (len(i) == 0 or i[0].state == "shutting-down" or i[0].state =="terminated" or i[0].state =="stopped"):
+                    print "Found dead node: " + str(node)
+                    output = Fork.check_output(["knife", "node", "delete", node, "--yes"], stderr=None)
+                    print output
+                    output = Fork.check_output(["knife", "client", "delete", node, "--yes"], stderr=None)
+                    print output
+                    alarms = cw_obj.describe_alarms(alarm_name_prefix=node)
+                    print "Alarms for node: " + str(node) + " found = " + str(len(alarms)) 
+                    for a in alarms:
+                        a.delete()
+                else:
+                    print "Node: " + str(node) + " has state: " + str(i[0].state) 
+            #sleep to ensure we dont hit rate limit on aws
+            time.sleep(5)        
+
